@@ -1,16 +1,18 @@
 import logging
+import pdb
+
 import tensorflow as tf
 from tensorflow.keras.layers import Activation
 from typing import Union, Callable, Optional
 from tensorflow import Tensor
 from numpy import pi
-import cvnn
+# import cvnn
 
 """
 This module contains many complex-valued activation functions to be used by CVNN class.
 """
 
-logger = logging.getLogger(cvnn.__name__)
+# logger = logging.getLogger(cvnn.__name__)
 t_activation = Union[str, Callable]  # TODO: define better
 
 
@@ -24,6 +26,55 @@ def linear(z: Tensor) -> Tensor:
     return z
 
 
+def modrelu(z: Tensor, b: float = 1., c: float = 1e-3) -> Tensor:
+    """
+    mod ReLU presented in "Unitary Evolution Recurrent Neural Networks"
+        from M. Arjovsky et al. (2016)
+        URL: https://arxiv.org/abs/1511.06464
+    A variation of the ReLU named modReLU. It is a pointwise nonlinearity,
+    modReLU(z) : C -> C, which affects only the absolute
+    value of a complex number, defined:
+        modReLU(z) = ReLU(|z|+b)*z/|z|
+    TODO: See how to check the non zero abs.
+    """
+    abs_z = tf.math.abs(z)
+    return tf.cast(tf.keras.activations.relu(abs_z + b), dtype=z.dtype) * z / tf.cast(abs_z + c, dtype=z.dtype)
+
+
+def zrelu(z: Tensor) -> Tensor:
+    """
+    zReLU presented in "On Complex Valued Convolutional Neural Networks"
+        from Nitzan Guberman (2016).
+    This methods let's the output as the input if both real and imaginary parts are positive.
+
+    https://stackoverflow.com/questions/49412717/advanced-custom-activation-function-in-keras-tensorflow
+    """
+    angle = tf.math.angle(z)
+    return tf.keras.backend.switch(0 <= angle,
+                                   tf.keras.backend.switch(angle <= pi / 2,
+                                                           z,
+                                                           tf.cast(0., dtype=z.dtype)),
+                                   tf.cast(0., dtype=z.dtype))
+
+
+def crelu(z: Tensor, alpha: float = 0.0, max_value: Optional[float] = None, threshold: float = 0) -> Tensor:
+    """
+    Mirror of cart_relu
+    """
+    return cart_relu(z, alpha, max_value, threshold)
+
+
+def complex_cardioid(z: Tensor) -> Tensor:
+    """
+    Complex cardioid presented in "Better than Real: Complex-valued Neural Nets for MRI Fingerprinting"
+        from V. Patrick (2017).
+        
+    This function maintains the phase information while attenuating the magnitude based on the phase itself. 
+    For real-valued inputs, it reduces to the ReLU.
+    """
+    return tf.cast(1 + tf.math.cos(tf.math.angle(z)), dtype=z.dtype) * z / 2.
+
+            
 """
 Complex input, real output
 """
@@ -40,7 +91,7 @@ def softmax_real_with_abs(z: Tensor, axis=-1) -> Tensor:
     It is often used as the activation for the last layer of a classification network because the result could be
     interpreted as a probability distribution.
     The softmax of x is calculated by exp(x)/tf.reduce_sum(exp(x)).
-    https://www.tensorflow.org/api_docs/python/tf/keras/activations/softmax
+        https://www.tensorflow.org/api_docs/python/tf/keras/activations/softmax
     :param z: Input tensor.
     :return: Real-valued tensor of the applied activation function
     """
@@ -52,18 +103,17 @@ def softmax_real_with_abs(z: Tensor, axis=-1) -> Tensor:
 
 def softmax_real_with_avg(z: Tensor, axis=-1) -> Tensor:
     """
-    Applies the softmax function to the modulus of z.
     The softmax activation function transforms the outputs so that all values are in range (0, 1) and sum to 1.
     It is often used as the activation for the last layer of a classification network because the result could be
     interpreted as a probability distribution.
     The softmax of x is calculated by exp(x)/tf.reduce_sum(exp(x)).
-    https://www.tensorflow.org/api_docs/python/tf/keras/activations/softmax
+        https://www.tensorflow.org/api_docs/python/tf/keras/activations/softmax
     :param z: Input tensor.
     :return: Real-valued tensor of the applied activation function
     """
     if z.dtype.is_complex:
-        return 0.5 * (tf.keras.activations.softmax(tf.math.real(z), axis) + tf.keras.activations.softmax(tf.math.real(z),
-                                                                                                        axis))
+        return 0.5 * (tf.keras.activations.softmax(tf.math.real(z), axis) + tf.keras.activations.softmax(
+            tf.math.real(z), axis))
     else:
         return tf.keras.activations.softmax(z, axis)
 
@@ -281,7 +331,8 @@ def cart_sigmoid(z: Tensor) -> Tensor:
     :return: Tensor result of the applied activation function
     """
     return tf.cast(tf.complex(tf.keras.activations.sigmoid(tf.math.real(z)),
-                              tf.keras.activations.sigmoid(tf.math.imag(z))), dtype=z.dtype)
+                              tf.keras.activations.sigmoid(tf.math.imag(z))), 
+                   dtype=z.dtype)
 
 
 def cart_elu(z: Tensor, alpha=1.0) -> Tensor:
@@ -521,10 +572,22 @@ act_dispatcher = {
     'etf_circular_sinh': Activation(etf_circular_sinh),
     'etf_inv_circular_atanh': Activation(etf_inv_circular_atanh),
     'etf_inv_circular_asinh': Activation(etf_inv_circular_asinh),
+    # ReLU
+    'modrelu': Activation(modrelu),
+    'crelu': Activation(crelu),
+    'zrelu': Activation(zrelu),
+    'complex_cardioid': Activation(complex_cardioid)
 }
 
 if __name__ == '__main__':
-    import matplotlib.pyplot as plt
+    x = tf.constant([-2, 1.0, 0.0, 1.0, -3, 0.8, 0.1], dtype=tf.float32)
+    y = tf.constant([-2.5, -1.5, 0.0, 1.0, 2, 0.4, -0.4], dtype=tf.float32)
+    z = tf.complex(x, y)
+    result = crelu(z)
+    result = modrelu(z, 4)
+    result = zrelu(z)
+    result = complex_cardioid(z)
+    """import matplotlib.pyplot as plt
     import numpy as np
 
     x = tf.constant([-2, 1.0, 0.0, 1.0, -3, 0.8, 0.1], dtype=tf.float32)
@@ -548,9 +611,10 @@ if __name__ == '__main__':
 
     ax.set_ylim(ymin=-axis_max, ymax=axis_max)
     ax.set_xlim(xmin=-axis_max, xmax=axis_max)
-    plt.show()
+    plt.show()"""
+    
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.0.16'
+__version__ = '0.0.19'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
