@@ -29,7 +29,7 @@ from tensorflow import data
 from typing import Type
 
 logger = logging.getLogger(cvnn.__name__)
-
+DEFAULT_OUTPUT_ACT = 'softmax_real_with_abs'
 t_path = Union[str, Path]
 
 
@@ -149,8 +149,8 @@ class MonteCarlo:
                                        epochs=epochs, batch_size=batch_size,
                                        verbose=debug, validation_freq=display_freq,
                                        callbacks=callbacks)
-                test_results = self._inner_callback(model, validation_data, confusion_matrix, real_cast_modes, i, run_result,
-                                                    test_results, test_data_fit, temp_path)
+                test_results = self._inner_callback(model, validation_data, confusion_matrix, real_cast_modes[i], i,
+                                                    run_result, test_results, test_data_fit, temp_path)
             self._outer_callback(pbar)
         return self._end_callback(x, y, iterations, data_summary, real_cast_modes, epochs, batch_size,
                                   confusion_matrix, test_results, pbar, w_save)
@@ -403,82 +403,6 @@ class RealVsComplex(MonteCarlo):
 # ====================================
 #   Monte Carlo simulation methods
 # ====================================
-def run_montecarlo(models: List[Model], dataset: cvnn.dataset.Dataset, open_dataset: Optional[t_path] = None,
-                   iterations: int = 30,
-                   epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
-                   validation_split: float = 0.2,
-                   validation_data: Optional[Union[Tuple, data.Dataset]] = None,  # TODO: Add vallidation data tuple details
-                   debug: bool = False, do_conf_mat: bool = False, do_all: bool = True, tensorboard: bool = False,
-                   polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
-                   plot_data: bool = False) -> str:
-    """
-    This function is used to compare different neural networks performance.
-    1. Runs simulation and compares them.
-    2. Saves several files into ./log/montecarlo/date/of/run/
-        2.1. run_summary.txt: Summary of the run models and data
-        2.2. run_data.csv: Full information of performance of iteration of each model at each epoch
-        2.3. <model_name>_statistical_result.csv: Statistical results of all iterations of each model per epoch
-        2.4. (Optional) `plot/` folder with the corresponding plots generated by MonteCarloAnalyzer.do_all()
-
-    :param models: List of cvnn.CvnnModel to be compared.
-    :param dataset: cvnn.dataset.Dataset with the dataset to be used on the training
-    :param open_dataset: (None)
-        If dataset is saved inside a folder and must be opened, path of the Dataset to be opened. Else None (default)
-    :param iterations: Number of iterations to be done for each model
-    :param epochs: Number of epochs for each iteration
-    :param batch_size: Batch size at each iteration
-    :param display_freq: Frequency in terms of epochs of when to do a checkpoint.
-    :param debug:
-    :param polar: Boolean weather the RVNN should receive real and imaginary part (False) or amplitude and phase (True)
-    :param do_all: If true (default) it creates a `plot/` folder with the plots generated by MonteCarloAnalyzer.do_all()
-    :param validation_split: Float between 0 and 1.
-            Percentage of the input data to be used as test set (the rest will be use as train set)
-            Default: 0.0 (No validation set).
-            This input is ignored if validation_data is given.
-        :param validation_data: Data on which to evaluate the loss and any model metrics at the end of each epoch.
-            The model will not be trained on this data. This parameter takes precedence over validation_split.
-            It can be:
-                - tuple (x_val, y_val) of Numpy arrays or tensors. Preferred data type (less overhead).
-                - A tf.data dataset.
-    :param do_conf_mat: Generate a confusion matrix based on results.
-    :return: (string) Full path to the run_data.csv generated file.
-        It can be used by cvnn.data_analysis.SeveralMonteCarloComparison to compare several runs.
-    """
-    if open_dataset:
-        dataset = dp.OpenDataset(open_dataset)  # Warning, open_dataset overwrites dataset
-
-    # Monte Carlo
-    monte_carlo = MonteCarlo()
-    for model in models:
-        # model.training_param_summary()
-        monte_carlo.add_model(model)
-    if not open_dataset:
-        dataset.save_data(monte_carlo.monte_carlo_analyzer.path)
-    monte_carlo.output_config['excel_summary'] = False
-    monte_carlo.output_config['tensorboard'] = tensorboard
-    monte_carlo.output_config['confusion_matrix'] = do_conf_mat
-    monte_carlo.output_config['plot_all'] = do_all
-    if plot_data:
-        dataset.plot_data(overlapped=True, showfig=False, save_path=monte_carlo.monte_carlo_analyzer.path,
-                          library='matplotlib')
-    monte_carlo.run(dataset.x, dataset.y, iterations=iterations,
-                    validation_split=validation_split, validation_data=validation_data,
-                    epochs=epochs, batch_size=batch_size, display_freq=display_freq,
-                    shuffle=False, debug=debug, data_summary=dataset.summary(), real_cast_modes=polar)
-
-    # Save data to remember later what I did.
-    _save_montecarlo_log(iterations=iterations,
-                         path=str(monte_carlo.monte_carlo_analyzer.path),
-                         models_names=[str(model.name) for model in models],
-                         dataset_name=dataset.dataset_name,
-                         num_classes=str(dataset.y.shape[1]),
-                         polar_mode=str(polar),
-                         dataset_size=str(dataset.x.shape[0]),
-                         features_size=str(dataset.x.shape[1]), epochs=epochs, batch_size=batch_size
-                         # filename='./log/run_data.csv'
-                         )
-    return str("./log/run_data.csv")
-
 
 def run_gaussian_dataset_montecarlo(iterations: int = 30, m: int = 10000, n: int = 128, param_list=None,
                                     epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
@@ -559,17 +483,95 @@ def run_gaussian_dataset_montecarlo(iterations: int = 30, m: int = 10000, n: int
                                                   plot_data=plot_data)
 
 
+def run_montecarlo(models: List[Model], dataset: cvnn.dataset.Dataset, open_dataset: Optional[t_path] = None,
+                   iterations: int = 30,
+                   epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
+                   validation_split: float = 0.2,
+                   validation_data: Optional[Union[Tuple, data.Dataset]] = None,  # TODO: Add vallidation data tuple details
+                   debug: Union[bool, int] = False, do_conf_mat: bool = False, do_all: bool = True, tensorboard: bool = False,
+                   polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
+                   plot_data: bool = False) -> str:
+    """
+    This function is used to compare different neural networks performance.
+    1. Runs simulation and compares them.
+    2. Saves several files into ./log/montecarlo/date/of/run/
+        2.1. run_summary.txt: Summary of the run models and data
+        2.2. run_data.csv: Full information of performance of iteration of each model at each epoch
+        2.3. <model_name>_statistical_result.csv: Statistical results of all iterations of each model per epoch
+        2.4. (Optional) `plot/` folder with the corresponding plots generated by MonteCarloAnalyzer.do_all()
+
+    :param models: List of cvnn.CvnnModel to be compared.
+    :param dataset: cvnn.dataset.Dataset with the dataset to be used on the training
+    :param open_dataset: (None)
+        If dataset is saved inside a folder and must be opened, path of the Dataset to be opened. Else None (default)
+    :param iterations: Number of iterations to be done for each model
+    :param epochs: Number of epochs for each iteration
+    :param batch_size: Batch size at each iteration
+    :param display_freq: Frequency in terms of epochs of when to do a checkpoint.
+    :param debug:
+    :param polar: Boolean weather the RVNN should receive real and imaginary part (False) or amplitude and phase (True)
+    :param do_all: If true (default) it creates a `plot/` folder with the plots generated by MonteCarloAnalyzer.do_all()
+    :param validation_split: Float between 0 and 1.
+            Percentage of the input data to be used as test set (the rest will be use as train set)
+            Default: 0.0 (No validation set).
+            This input is ignored if validation_data is given.
+        :param validation_data: Data on which to evaluate the loss and any model metrics at the end of each epoch.
+            The model will not be trained on this data. This parameter takes precedence over validation_split.
+            It can be:
+                - tuple (x_val, y_val) of Numpy arrays or tensors. Preferred data type (less overhead).
+                - A tf.data dataset.
+    :param do_conf_mat: Generate a confusion matrix based on results.
+    :return: (string) Full path to the run_data.csv generated file.
+        It can be used by cvnn.data_analysis.SeveralMonteCarloComparison to compare several runs.
+    """
+    if open_dataset:
+        dataset = dp.OpenDataset(open_dataset)  # Warning, open_dataset overwrites dataset
+
+    # Monte Carlo
+    monte_carlo = MonteCarlo()
+    for model in models:
+        # model.training_param_summary()
+        monte_carlo.add_model(model)
+    if not open_dataset:
+        dataset.save_data(monte_carlo.monte_carlo_analyzer.path)
+    monte_carlo.output_config['excel_summary'] = False
+    monte_carlo.output_config['tensorboard'] = tensorboard
+    monte_carlo.output_config['confusion_matrix'] = do_conf_mat
+    monte_carlo.output_config['plot_all'] = do_all
+    if plot_data:
+        dataset.plot_data(overlapped=True, showfig=False, save_path=monte_carlo.monte_carlo_analyzer.path,
+                          library='matplotlib')
+    monte_carlo.run(dataset.x, dataset.y, iterations=iterations,
+                    validation_split=validation_split, validation_data=validation_data,
+                    epochs=epochs, batch_size=batch_size, display_freq=display_freq,
+                    shuffle=False, debug=debug, data_summary=dataset.summary(), real_cast_modes=polar)
+
+    # Save data to remember later what I did.
+    _save_montecarlo_log(iterations=iterations,
+                         path=str(monte_carlo.monte_carlo_analyzer.path),
+                         models_names=[str(model.name) for model in models],
+                         dataset_name=dataset.dataset_name,
+                         num_classes=str(dataset.y.shape[1]),
+                         polar_mode=str(polar),
+                         dataset_size=str(dataset.x.shape[0]),
+                         features_size=str(dataset.x.shape[1]), epochs=epochs, batch_size=batch_size
+                         # filename='./log/run_data.csv'
+                         )
+    return str("./log/run_data.csv")
+
+
 def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_dataset: Optional[t_path] = None,
                                        iterations: int = 30,
                                        epochs: int = 300, batch_size: int = 100, display_freq: int = 1,
                                        optimizer='adam',     # TODO: Typing
                                        shape_raw=None, activation: t_activation = 'cart_relu',
-                                       debug:  bool = False, do_all: bool = True,
+                                       output_activation : t_activation = DEFAULT_OUTPUT_ACT,
+                                       debug:  Union[bool, int] = False, do_all: bool = True,
                                        polar: Optional[Union[str, List[Optional[str]], Tuple[Optional[str]]]] = None,
                                        dropout: float = 0.5, validation_split: float = 0.2,
                                        validation_data: Optional[Union[Tuple, data.Dataset]] = None,    # TODO: Add typing of tuple
                                        capacity_equivalent: bool = True, equiv_technique: str = 'ratio',
-                                       shuffle: bool = False, tensorboard: bool = False,
+                                       shuffle: bool = False, tensorboard: bool = False, do_conf_mat: bool = False,
                                        plot_data: bool = True) -> str:
     """
     This function is used to compare CVNN vs RVNN performance over any dataset.
@@ -603,11 +605,11 @@ def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_datas
             Percentage of the input data to be used as test set (the rest will be use as train set)
             Default: 0.0 (No validation set).
             This input is ignored if validation_data is given.
-        :param validation_data: Data on which to evaluate the loss and any model metrics at the end of each epoch.
-            The model will not be trained on this data. This parameter takes precedence over validation_split.
-            It can be:
-                - tuple (x_val, y_val) of Numpy arrays or tensors. Preferred data type (less overhead).
-                - A tf.data dataset.
+    :param validation_data: Data on which to evaluate the loss and any model metrics at the end of each epoch.
+        The model will not be trained on this data. This parameter takes precedence over validation_split.
+        It can be:
+            - tuple (x_val, y_val) of Numpy arrays or tensors. Preferred data type (less overhead).
+            - A tf.data dataset.
     :param capacity_equivalent: An equivalent model can be equivalent in terms of layer neurons or
                         trainable parameters (capacity equivalent according to: https://arxiv.org/abs/1811.12351)
             - True, it creates a capacity-equivalent model in terms of trainable parameters
@@ -625,26 +627,11 @@ def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_datas
         shape_raw = [64]
     if open_dataset:
         dataset = dp.OpenDataset(open_dataset)  # Warning, open_dataset overwrites dataset
-    # Create complex network
     input_size = dataset.x.shape[1]  # Size of input
     output_size = dataset.y.shape[1]  # Size of output
-    shape = [
-        layers.ComplexInput(input_shape=input_size)
-    ]
-    if len(shape_raw) == 0:
-        logger.warning("No hidden layers are used. activation and dropout will be ignored")
-        shape.append(
-            ComplexDense(units=output_size, activation='softmax_real', input_dtype=np.complex64)
-        )
-    else:  # len(shape_raw) > 0:
-        for s in shape_raw:
-            shape.append(ComplexDense(units=s, activation=activation))   # Add dropout!
-            if dropout is not None:
-                shape.append(ComplexDropout(rate=dropout))
-        shape.append(ComplexDense(units=output_size, activation='softmax_real'))
-
-    complex_network = tf.keras.Sequential(shape, name="complex_network")
-    complex_network.compile(optimizer=optimizer, loss=tf.keras.losses.CategoricalCrossentropy(), metrics=['accuracy'])
+    complex_network = get_mlp(input_size=input_size, output_size=output_size,
+                              shape_raw=shape_raw, activation=activation, dropout=dropout,
+                              output_activation=output_activation, optimizer=optimizer)
 
     # Monte Carlo
     monte_carlo = RealVsComplex(complex_network,
@@ -653,6 +640,7 @@ def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_datas
     # monte_carlo.output_config['confusion_matrix'] = do_conf_mat
     monte_carlo.output_config['plot_all'] = do_all
     monte_carlo.output_config['excel_summary'] = False
+    monte_carlo.output_config['confusion_matrix'] = do_conf_mat
     if plot_data:
         dataset.plot_data(overlapped=True, showfig=False, save_path=monte_carlo.monte_carlo_analyzer.path,
                           library='matplotlib')
@@ -713,6 +701,30 @@ def mlp_run_real_comparison_montecarlo(dataset: cvnn.dataset.Dataset, open_datas
     return str(monte_carlo.monte_carlo_analyzer.path / "run_data.csv")
 
 
+def get_mlp(input_size, output_size,
+            shape_raw=None, activation="cart_relu", dropout=0.5,
+            output_activation='softmax_real_with_abs', optimizer="sgd", name="complex_network"):
+    if shape_raw is None:
+        shape_raw = [100, 50]
+    shape = [
+        layers.ComplexInput(input_shape=input_size)
+    ]
+    if len(shape_raw) == 0:
+        shape.append(
+            ComplexDense(units=output_size, activation=output_activation, input_dtype=np.complex64)
+        )
+    else:  # len(shape_raw) > 0:
+        for s in shape_raw:
+            shape.append(ComplexDense(units=s, activation=activation))  # Add dropout!
+            if dropout is not None:
+                shape.append(ComplexDropout(rate=dropout))
+        shape.append(ComplexDense(units=output_size, activation=output_activation))
+
+    complex_network = tf.keras.Sequential(shape, name=name)
+    complex_network.compile(optimizer=optimizer, loss=tf.keras.losses.CategoricalCrossentropy(), metrics=['accuracy'])
+    return complex_network
+
+
 # ====================================
 #     Excel logging
 # ====================================
